class Events:
    connect: str = "connect"
    disconnect: str = "disconnect"
    error: str = "error"
    state_update: str = "state-update"
    playlist_created: str = "playlist-created"
    playlist_deleted: str = "playlist-deleted"