state_converter = {
    -1: "Unknown",
    0: "Paused",
    1: "Playing",
    2: "Buffering"
}

repeatMode_converter = {
    -1: "Unknown",
    0: "None",
    1: "All",
    2: "One"
}

like_status_converter = {
    -1: "Unknown",
    0: "Dislike",
    1: "Indifferent",
    2: "Like"
}